/*
   Billboard.cpp

   Written by Zero (xmaste00).

   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.
   You may use, modify or distribute it freely.

   This source code is distributed in the hope that it will be useful but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   If you find the source code useful, authors will kindly welcome
   if you give them credit and keep their names with their source code,
   but you are not forced to do so.
*/
#include "Billboard.h"

void Billboard::updateDimension(float dimension){
	float Vertices[4][3] = {
		-this->Dimension,-this->Dimension,0,
		+this->Dimension,-this->Dimension,0,
		-this->Dimension,+this->Dimension,0,
		+this->Dimension,+this->Dimension,0,
	};
	this->Coordinate->point.setValues(0,4,Vertices);
}
void Billboard::updateTextureCoordinate(int frame){
	float x0=1.0f/(float)(TextureDimension)*(frame%TextureDimension);
	float y0=1.0f/(float)(TextureDimension)*(frame/TextureDimension);
	float x1=1.0f/(float)(TextureDimension)*(frame%TextureDimension+1);
	float y1=1.0f/(float)(TextureDimension)*(frame/TextureDimension+1);
	TextureCoordinate->point.set1Value(0,x0,y0);
	TextureCoordinate->point.set1Value(1,x1,y0);
	TextureCoordinate->point.set1Value(2,x0,y1);
	TextureCoordinate->point.set1Value(3,x1,y1);
}

Billboard::Billboard(){
	Root = new SoSeparator;
	Root->ref();

	Rotation = new SoRotation;
	Rotation->rotation.setValue(SbVec3f(0,1,0),0);
	Root->addChild(Rotation);

	Translation = new SoTranslation;
	Translation->translation.setValue(0,0,0);
	Root->addChild(Translation);

	this->Dimension=0.5f;
	Coordinate = new SoCoordinate3;
	this->updateDimension(this->Dimension);
	Root->addChild(Coordinate);

	SoTransparencyType * TransparensyType = new SoTransparencyType;
	TransparensyType->value = SoTransparencyType::DELAYED_BLEND; //DELAYED_ADD DELAYED_BLEND
	Root->addChild(TransparensyType);

	Texture = new SoTexture2;
	Texture->model.setValue( SoTexture2::MODULATE);  //MODULATE DECAL BLEND REPLACE 
	Root->addChild(Texture);

	TextureCoordinate = new SoTextureCoordinate2;
	TextureCoordinate->point.set1Value(0,0,0);
	TextureCoordinate->point.set1Value(1,0,1);
	TextureCoordinate->point.set1Value(2,1,1);
	TextureCoordinate->point.set1Value(3,1,0);
	Root->addChild(TextureCoordinate);

	int32_t Indices[5] = {0,1,2,3,-1};
	int32_t TextureIndex[5] = {0,1,2,3,-1};
	TriangleStripSet = new SoIndexedTriangleStripSet;
	TriangleStripSet->coordIndex.setValues(0,5,Indices);
	TriangleStripSet->textureCoordIndex.setValues(0,5,TextureIndex);
	Root->addChild(TriangleStripSet);

	this->FPS=15;
	this->Used=false;
	this->TextureDimension=1;
	this->Frame=0;
	this->lastTime=0;

	this->Camera = NULL;
}
Billboard::Billboard(float dimension, char * file){
	Billboard();
	this->Dimension=dimension;
	this->updateDimension(dimension);
	this->Texture->filename.setValue(file);
}
void Billboard::useBillboard(){
	this->Used=true;
	this->Frame=0;
	this->lastTime=0;
}
void Billboard::useBillboard(SbVec3f pos){
	this->Used=true;
	this->Frame=0;
	this->lastTime=0;
	this->Translation->translation.setValue(pos);
	this->updateTextureCoordinate(this->Frame);
}
void Billboard::timeStep(double time, double dt){
	if (this->Camera!=NULL) this->Rotation->rotation.setValue(this->Camera->orientation.getValue());
	if ((time-this->lastTime)>(1.0f/this->FPS)){
		this->lastTime=time;
		if (this->Frame >= (this->TextureDimension*this->TextureDimension)) {
			this->Frame=0;
			this->Used=false;
			this->Manager->removeChild(this->Root);
		} else {
		    this->updateTextureCoordinate(this->Frame);
		    this->Frame++;
        }
	}
}