/*

   SgObject.cpp - base class for scene objects

   Written by Petr Mastera (xmaste00).


   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.
   You may use, modify or distribute it freely.

   This source code is distributed in the hope that it will be useful but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   If you find the source code useful, authors will kindly welcome
   if you give them credit and keep their names with their source code,
   but you are not forced to do so.

*/

#include "SgObject.h"

SgObject::SgObject() : Manager(NULL), CollisionRadius(1.0f), ActionRadius(1.0f), Mass(1.0f), CoefficientOfRestitution(1.0f),
                       Position(0.0f,0.0f,0.0f), Velocity(0.f,0.f,0.f), AngularVelocity(0.f,0.f,0.f), AngularMomentum(0.f,0.f,0.f),
					   Force(0.0f,0.0f,0.0f), ID(0), Type(OTHER), Select(false), durFB(0), durLR(0), durUD(0), durX(0), durY(0), durZ(0) 

{
	this->InertiaTensor.setValue(SbMatrix((3/(this->Mass*(2.572816f+7.76458225f))),0,0,0,
    									   0,(3/(this->Mass*(6.76f+7.76458225f))),0,0,
										   0,0,(3/(this->Mass*(6.76f+2.572816f))),0,
										   0,0,0,1));
  createHiddenScene();
}
SgObject::SgObject(const char * model, const char * collisionModel,
                   const SbVec3f & pos, const SbVec3f & speed,
                   float radius, float mass)
{
  this->Manager=NULL;
  createHiddenScene();
  this->setModel(model,RENDER);
  this->setModel(collisionModel,COLLISION);
  this->CollisionRadius = radius;
  this->ActionRadius = radius;
  this->ID = 0;
  this->Type = OTHER;
  this->Mass = mass;
  this->Position = pos;
  this->Velocity = speed;
  this->AngularVelocity = SbVec3f(0,0,0);
  this->AngularMomentum = SbVec3f(0,0,0);
  this->Force = SbVec3f(0.0f,0.0f,0.0f);
  this->Select = false;
  this->CoefficientOfRestitution=1.0f;
  this->durFB=0;
  this->durLR=0;
  this->durUD=0;
  this->durX=0;
  this->durY=0;
  this->durZ=0;
}
SgObject::~SgObject()
{
  if (Manager) Manager->removeSgo(this);
  releaseHiddenScene();
}
void SgObject::createHiddenScene()
{
  Root = new SoSeparator;
  Root->ref();
  MatrixTransform = new SoMatrixTransform;
  Root->addChild(MatrixTransform);
  Model = NULL;
  CollisionModel = NULL;

  SbMatrix m(MatrixTransform->matrix.getValue());
  m.getTransform(this->Position,this->Orientation,this->Scale,this->ScaleOrientation);

  //SoRayPickAction RPA = new SoRayPickAction(Root->get);

}
void SgObject::releaseHiddenScene()
{
  Root->unref();
  if (CollisionModel) CollisionModel->unref();
#ifndef NDEBUG // debugging purposes only
  Root = NULL;
  MatrixTransform = NULL;
  Model = NULL;
  CollisionModel = NULL;
#endif
}
void SgObject::updateMatrixTransform(){
  SbMatrix m;
  m.setTransform(this->Position, this->Orientation, this->Scale, this->ScaleOrientation);
  this->MatrixTransform->matrix.setValue(m);
}
void SgObject::setModel(SoNode * model, MODEL_SET which)
{
  if ((which & RENDER) != 0) {
    if (this->Model) Root->removeChild(this->Model);
    this->Model = model;
    if (model) Root->addChild(model);
  }
  if ((which & COLLISION) != 0) {
    if (this->CollisionModel) this->CollisionModel->unref();
    this->CollisionModel = model;
    if (this->CollisionModel) this->CollisionModel->ref();
  }
}
SbBool SgObject::setModel(const char *filename, MODEL_SET which)
{
  if (!which) return TRUE;
  SoInput in;
  if (!in.openFile(filename)) return FALSE;
  SoSeparator * model = SoDB::readAll(&in);
  model->ref();
  setModel(model, which);
  model->unref();
  return TRUE;
}

SbVec3f SgObject::getDirVel() const
{
	SbVec3f dirvel=this->Velocity;
	if (dirvel.length()) dirvel.normalize();
	return dirvel;
}
SbVec3f SgObject::getDirVec() const
{
	SbVec3f dirvec(0, 0, 1);
	this->Orientation.multVec(dirvec,dirvec);
	dirvec.normalize();
	return dirvec;
}
SbVec3f SgObject::getUpVec() const
{
	SbVec3f upvec(0, 1, 0);
	this->Orientation.multVec(upvec,upvec);
	upvec.normalize();
	return upvec;
}
SbVec3f SgObject::getLeftVec() const
{
	SbVec3f leftvec(-1, 0, 0);
	this->Orientation.multVec(leftvec,leftvec);
	leftvec.normalize();
	return leftvec;
}
void SgObject::timeStep(double time, double dt)
{
	if (this->Select){
	};
	this->OldPosition = this->Position;
	this->OldOrientation = this->Orientation;
	float size;

	if (this->Type!=this->SUN) this->Velocity+=this->Force;
	this->updPos(this->Velocity * float(dt));
	SbVec3f r=this->Velocity;
	this->AngularVelocity=this->AngularMomentum;
	size=this->AngularVelocity.length();
	if (size){
		//this->AngularVelocity.normalize();
		this->updOrientation(this->AngularVelocity,(size)*float(dt));
	}
	SbMatrix m,mt;
	this->InertiaTensor.getValue(m);
/*	cout<<"inertia tensor"<<endl;
	float f[4];
	float *p;
	p=m[0];
	cout<<*p++<<" "<<*p++<<" "<<*p++<<" "<<*p++<<endl;
	p=m[1];
	cout<<*p++<<" "<<*p++<<" "<<*p++<<" "<<*p++<<endl;
	p=m[2];
	cout<<*p++<<" "<<*p++<<" "<<*p++<<" "<<*p++<<endl;
	p=m[3];
	cout<<*p++<<" "<<*p++<<" "<<*p++<<" "<<*p++<<endl;
	cout<<endl;*/
	this->Orientation.getValue(m);
	mt=m.transpose();
	this->InertiaTensor=m*this->InertiaTensor*mt;
	this->InertiaTensor.getValue(m);
	/*cout<<"inertia tensor"<<endl;
	p=m[0];
	cout<<*p++<<" "<<*p++<<" "<<*p++<<" "<<*p++<<endl;
	p=m[1];
	cout<<*p++<<" "<<*p++<<" "<<*p++<<" "<<*p++<<endl;
	p=m[2];
	cout<<*p++<<" "<<*p++<<" "<<*p++<<" "<<*p++<<endl;
	p=m[3];
	cout<<*p++<<" "<<*p++<<" "<<*p++<<" "<<*p++<<endl;
	cout<<endl;*/
	//cout<<"determinant: "<<this->InertiaTensor.det3()<<endl;
  this->updateMatrixTransform();
}