#ifndef SGO_MANAGER_H
#define SGO_MANAGER_H
/*

   SgoManager.h - scene objects manager class

   Written by Petr Mastera (xmaste00) and PCJohn (peciva).


   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.
   You may use, modify or distribute it freely.

   This source code is distributed in the hope that it will be useful but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   If you find the source code useful, authors will kindly welcome
   if you give them credit and keep their names with their source code,
   but you are not forced to do so.

*/
#include "Main.h"

class SgObject;
class Billboard;

/**  SgoManager class.
 * SgoManager class manages the set of SgObjects.
 *
 * The word SgoManager is abbreviation of SpaceGame Object Manager.
 */

class SgoManager {
public:
  #define G 6.67e-11f
  enum CollidingType { Null=0, Sphere, Body, Explosion };

private:
  SoSeparator * sgoRoot;       ///< Scene root.
  SbList<SgObject*> sgoList;  ///< List of space game objects.
  SbList<Billboard*> sgoBillboard;

  SgObject * Active;
  int CameraFree;
  int CameraFighter;
  SoPerspectiveCamera * Camera;
  SoGroup * collisionRoot;     ///< SoGroup used for collision detection.


  SoIntersectionDetectionAction * ida;  ///< Collision class.
  static SoIntersectionDetectionAction::Resp intersectionCb(void *closure,
      const SoIntersectingPrimitive *p1, const SoIntersectingPrimitive *p2); ///< Collision callback.
  SgoManager::CollidingType checkCollisionWith(SgObject * sgo, int index);
  SgoManager::CollidingType checkCollision();
  void resolveCollisionExplozion(const double time, const double dt);
  void resolveCollisionSphere();
  void resolveCollisionBody();
  
  float diffExp2(SbVec3f v1, SbVec3f v2);
  void workIntersectionPoints();
  void workIntersectionVertexs();
  SbVec3f getIntersectionPoint();
  void computeNormal();
  
  
  float epsilon;

  struct{	// uchovava index do pole objektu a jeho kolidujici trojuhelniky
	  bool Collision;
	  int Index[2];
	  //SoNode * Node[2];
	  vector<SbVec3f> Points;
	  vector<SbVec3f> Triangles[2];
	  SbVec3f collidingPoint;
	  SbVec3f collidingNormal;
  } CollidingSgos;

public:
  SgoManager();
  ~SgoManager();


  setFreeCamera(int i) { this->CameraFree=i; }
  setFighterCamera(int i) { this->CameraFighter=i; }
  setActiveFighter() { this->Active=this->sgoList[CameraFighter]; }
  setActiveFree() { this->Active=this->sgoList[CameraFree]; }

  SoSeparator * getSgoRoot() const { return sgoRoot; }

  void setCamera(SoPerspectiveCamera * camera) { this->Camera=camera; }

  SgObject * getSgo(int index)  { return sgoList[index]; }
  const SgObject* getSgo(int index) const  { return sgoList[index]; }
  int getSgosNum() const  { return sgoList.getLength(); }

  SgObject * getActive() { return this->Active; }
	
  void appendSgo(SgObject *sgo);
  void removeSgo(SgObject *sgo);
  void removeSgo(int index);
  void deleteSgo(int index);
  void deleteSgo(SgObject *sgo);
  void deteleAllSgos();
  
  void timeStep(const double time, const double dt);
};

#endif /* SGO_MANAGER_H */
